clear all
set matsize 800
set more off

*DEFINE YOUR OWN PATH !!!!
global PATH = "............delmast_INSULARITY\delmast_STATA\Cultural_distance\linguistic_distance"

global PATHdata="$PATH\data"
global PATHtemp="$PATH\temp"
global PATHoutput="$PATH\output"



********************************************************
************************ STEP 1 ************************
********************************************************
* From the language tree (languages.dta) to a file with the # of common branches (combranches.dta)

use "$PATHdata/languages.dta", clear
gen num=_n
gen _=.
gen pippo=num
reshape wide _, i(num) j(pippo)
save "$PATHtemp/ciao", replace

use "$PATHtemp/ciao", clear
mkmat var*, matrix(A)
global numb_digits=15
sum num
global n=r(max)

/* this loop counts the number of common nodes between each language pair */
local l=1
while `l' <= $n {

local x=`l'
while `x' <= $n {
dis in yellow "lang `l' vs lang `x'"
replace _`x'=1 if el(A,`l',1)==el(A,`x',1) & num==`l'

local i=1
while `i'<$numb_digits {
	local j=`i'+1
	mkmat _`x' in `l', matrix(B) 
	qui replace _`x'=`j' if el(A,`l',`j')==el(A,`x',`j') & num==`l' & el(B,1,1)==`i'
				
local i=`i'+1			

}


local x=`x'+1
        }

local l=`l'+1
        }

		
preserve
keep  ethn_code num
sort num
save "$PATHoutput/lang_codes", replace
restore	

drop var*
order ethn_code num
rename num numA 
rename ethn_code codeA
reshape long _, i(num) j(numB)
drop if numA>numB 
rename _ combranches
replace combranches=0 if combranches==.

rename numB num
sort num
merge num using	"$PATHoutput/lang_codes"	
drop _merge
rename ethn_code codeB
drop num*
order codeA codeB
sort codeA codeB
save "$PATHoutput/hello", replace	

use "$PATHoutput/hello", clear
sort codeB codeA
rename codeB ciao
rename codeA codeB
rename ciao codeA
order codeA codeB
drop if codeA==codeB
append using "$PATHoutput/hello"

rename codeA lang_codeA
rename codeB lang_codeB
sort  lang_codeA lang_codeB
save "$PATHoutput/combranches.dta", replace	



********************************************************
************************ STEP 2 ************************
********************************************************
*ASSIGN LANGUAGES TO REGIONS

use "$PATHdata\delmast_database.dta", clear
bys region: egen sample_language=total(speakers)
sort country
by country: egen tot_sample_language=total(speakers)
gen language_regshare= (speakers/sample_language)*100 
egen langcode = group(language country) 
egen regcode = group(reg_cod) 
tostring regcode, replace

preserve
keep regcode reg_cod
contract reg_cod regcode
drop _
sort regcode
save "$PATHtemp\regcode+reg_cod.dta", replace 
restore

egen langcode2 = group(lang_code regcode)
keep country region regcode lang_code langcode2 language_regshare Island 
save "$PATHtemp\ethnologue+reg.dta", replace

collapse (first) country region regcode lang_code (sum) language_regshare (mean) Island, by (langcode2)
save "$PATHoutput/eth+reg+clean.dta", replace

keep regcode  lang_code
gen b=_n
sort b
order regcode
save "$PATHtemp/ethnologue+reg+n.dta", replace

use "$PATHtemp\ethnologue+reg.dta", clear
keep country region regcode 
contract country region regcode 
drop _f
save "$PATHtemp\regions.dta", replace

use "$PATHtemp\regions.dta", clear
# delimit ;
forvalues i=1(1)478 {;
qui use "$PATHtemp/ethnologue+reg+n.dta", clear;
qui keep if regcode=="`i'";
qui gen a=_n;
qui sort a;
qui save "$PATHtemp/luca1", replace;
qui append using "$PATHtemp/ethnologue+reg+n.dta";
qui collapse a b, by(regcode lang_code);
qui fillin a b; 
qui drop if a==.;
qui drop regcode* lang_code* _fillin;
qui merge a using "$PATHtemp/luca1";
qui rename _merge _mergeA;
qui rename regcode regcodeA;
qui rename lang_code lang_codeA;
qui sort b;
qui merge b using "$PATHtemp/ethnologue+reg+n.dta";
qui rename _merge _mergeB;
qui rename regcode regcodeB;
qui rename lang_code lang_codeB;
qui keep regcode* lang_code*;
save "$PATHtemp/C-L_pairs_`i'.dta", replace;
	 };
#delimit cr

# delimit ;
forvalues i=1(1)478 {;
qui use "$PATHtemp/C-L_pairs_`i'.dta", clear;
qui drop if regcodeA==regcodeB; 
qui sort lang_codeA lang_codeB;
qui merge lang_codeA lang_codeB using "$PATHoutput/combranches";
qui keep if _merge==3; 
qui drop _m;
qui sort regcodeA lang_codeA  regcodeB  lang_codeB;
save "$PATHtemp/good_`i'", replace;
	 };
#delimit cr



********************************************************
************************ STEP 3 ************************
********************************************************
*COMPUTE THE INDEX

use "$PATHoutput\eth+reg+clean.dta", clear 
gen Q= language_regshare/100
keep regcode  lang_code Q 
sort  regcode lang_code Q
save "$PATHtemp/region+code+Q.dta", replace 
# delimit ;
forvalues i=1(1)478 {;
qui use "$PATHtemp/good_`i'.dta", clear;
qui rename regcodeA regcode;
qui rename lang_codeA lang_code;
qui sort  regcode lang_code;
qui merge regcode lang_code using "$PATHtemp/region+code+Q.dta";
qui keep if _m==3;
qui drop _m;
qui rename regcode regcodeA ;
qui rename lang_code lang_codeA ;
qui rename Q QA;
qui rename regcodeB regcode;
qui rename lang_codeB lang_code;
qui sort  regcode lang_code;
qui merge regcode lang_code using "$PATHtemp/region+code+Q.dta";
qui keep if _m==3;
qui drop _m;
qui rename regcode regcodeB;
qui rename lang_code lang_codeB;
qui rename Q QB;
qui gen dist_AB=1-(combranches/15)^0.5;
qui gen index_lang=QA*QB*dist_AB;
bys regcodeA regcodeB: egen index_final=sum(index_lang);
qui order regcode*;
qui collapse index_final, by (regcodeA regcodeB);
qui sort regcodeA regcodeB;
save "$PATHtemp/language_INDEX_`i'", replace;
	 };
#delimit cr

# delimit ;
use "$PATHtemp/language_INDEX_1", clear;
forvalues i=2(1)478 {;
append using "$PATHtemp/language_INDEX_`i'";
	 };
#delimit cr
save "$PATHtemp/language_INDEX_junk", replace 

use "$PATHtemp\regions.dta", clear
rename regcode regcodeA
rename country countryA
rename region regionA
sort regcodeA
merge 1:m regcodeA using "$PATHtemp/language_INDEX_junk.dta"
drop _m
sort regcodeB
save "$PATHtemp/junkjunk", replace
use "$PATHtemp\regions.dta", clear
rename regcode regcodeB
rename country countryB
rename region regionB
sort regcodeB
merge 1:m regcodeB using "$PATHtemp/junkjunk"
drop _m
order regcodeA regcodeB countryA regionA countryB regionB
sort regcodeA regcodeB
save "$PATHtemp/language_INDEX", replace

use "$PATHtemp/language_INDEX", clear
rename regcodeA regcode
merge m:1 regcode using "$PATHtemp\regcode+reg_cod.dta", nogenerate
rename regcode regcodeA
rename  reg_cod  reg_codA
rename regcodeB regcode
merge m:1 regcode using "$PATHtemp\regcode+reg_cod.dta", nogenerate
rename regcode regcodeB
rename  reg_code  reg_codB

drop if  reg_codA == "unk408"  // Gibraltar
drop if  reg_codA == "fra121"  //Guyane
drop if  reg_codA == "aus1"    //Norfolk island
drop if  reg_codA == "net301"  //Caribbean Netherlands */
drop if  reg_codB == "unk408"  // Gibraltar
drop if  reg_codB == "fra121"  //Guyane
drop if  reg_codB == "aus1"    //Norfolk island
drop if  reg_codB == "net301"  //Caribbean Netherlands */

save "$PATHoutput/language_INDEX_FINAL", replace









